require 'active_resource'
require 'awesome_print'

require 'activeresource/hel/version'
begin
require 'activeresource/hel/rails'
require 'activeresource/hel/railtie'
rescue NameError => ex
  # Rails unavailable?
end
require 'activeresource/hel/compat/formats'

module ActiveResource
  module Hel

    autoload :Connection,         'activeresource/hel/connection'
    autoload :Resource,           'activeresource/hel/resource'
    autoload :EmbeddedResource,   'activeresource/hel/embedded_resource'
    autoload :Collection,         'activeresource/hel/collection'
    autoload :EmbeddedCollection, 'activeresource/hel/embedded_collection'
    autoload :Session,            'activeresource/hel/session'
    autoload :Scopes,             'activeresource/hel/scopes'
    autoload :DefaultScopes,      'activeresource/hel/default_scopes'
    autoload :RemotePagination,   'activeresource/hel/remote_pagination'
    autoload :RemoteSchema,       'activeresource/hel/remote_schema'
    autoload :Validations,        'activeresource/hel/validations'
    autoload :Cached,             'activeresource/hel/cached'
    autoload :Dirty,              'activeresource/hel/dirty'
    autoload :Filters,            'activeresource/hel/filters'

    module Extensions
      autoload :Authentication,   'activeresource/hel/extensions/authentication'
      autoload :Cache,            'activeresource/hel/extensions/cache'
      autoload :Debugging,        'activeresource/hel/extensions/debugging'
    end

    module Cache
      autoload :Configuration,    'activeresource/hel/cache/configuration'
      autoload :Store,            'activeresource/hel/cache/store'
    end
  end

  class Connection
   # include ActiveResource::Hel::Extensions::Debugging
  end

end

module Ygg
  autoload :Session,  'ygg/session'
  module Core
    autoload :Identity,  'ygg/core/identity'
    autoload :Person,    'ygg/core/person'
  end
end

require 'active_resource/validations'
module ActiveResource
  class Errors < ActiveModel::Errors

    # monkey-patch validation looking into attributes.keys instead of known_attributes
    # since we are using active_model/dirty we are not sending all the attributes
    def from_hash(messages, save_cache = false)
      clear unless save_cache
      messages.each do |(key,errors)|
        errors.each do |error|
          if @base.known_attributes.include?(key)
            add key, error
          elsif key == 'base'
            self[:base] << error
          else
            # reporting an error on an attribute not in attributes
            # format and add them to base
            self[:base] << "#{key.humanize} #{error}"
          end
        end
      end
    end

  end
end
