require 'active_support/core_ext/module/delegation'

module ActiveResource::Hel
  module Cached
    extend ActiveSupport::Concern

    included do
      class << self
        threadsafe_attribute :_cached

        prepend PrependClassMethods
      end

      after_save :increase_request_version
      after_destroy :increase_request_version
      self.cached = true
    end

    module PrependClassMethods
      def find(*arguments)
        scope   = arguments.slice!(0)
        options = arguments.slice!(0) || {}
        
        revalidate = options.delete(:revalidate)
        if revalidate
          case scope
          when :all, :first, :last; :one
            force_revalidate_every(options.dup)
          else
            force_revalidate_single(scope, options.dup)
          end 
        end

        super(scope, options)
      end

      def post(custom_method_name, options = {}, body = "")
        increase_request_version
        super
      end

      def patch(custom_method_name, options = {}, body = "")
        increase_request_version
        super
      end

      def put(custom_method_name, options = {}, body = "")
        increase_request_version
        super
      end

      def delete(custom_method_name, options = {})
        increase_request_version
        super
      end
    end

    module ClassMethods
      def cached
        if _cached_defined?
          _cached
        elsif superclass != Object && superclass.cached
          superclass.cached
        end
      end

      def cached=(cached)
        self._connection = nil
        self._cached = cached
      end

      def force_revalidate_every(options)
        prefix_options, query_options = split_options(options[:params])
        key = collection_path(prefix_options)
        ActiveResource::Hel::Resource.cache.write("#{key}-cache-version", Time.new.to_i) 
      end

      def force_revalidate_single(scope, options)
        prefix_options, query_options = split_options(options[:params])
        key = element_path(scope, prefix_options)
        ActiveResource::Hel::Resource.cache.write("#{key}-cache-version", Time.new.to_i) 
      end

      def without_cache(&block)
        backup = self.cached
        self.cached = nil
        yield
      ensure
        self.cached = backup
      end

      def cache_version_key
        collection_path 
      end

      def increase_request_version
        ver = ActiveResource::Hel::Resource.cache.read("#{self.cache_version_key}-cache-version") || 0
        ActiveResource::Hel::Resource.cache.write("#{self.cache_version_key}-cache-version", ver + 1)
      end
    end

    def reload
      self.load(self.class.find(to_param, revalidate: true, params: @prefix_options).attributes, false, true)
    end


    protected

    def increase_request_version
      self.class.increase_request_version
    end
  end
end
