module ActiveResource::Hel
  module RemoteSchema
    extend ActiveSupport::Concern

    included do 
      class_attribute :remote_schema
      ActiveResource::Schema::KNOWN_ATTRIBUTE_TYPES.concat(['embedded_collection', 'collection'])
    end

    module ClassMethods

      #
      #
      #
      def load_remote_schema
        if defined?(@load_remote_schema)
          @load_remote_schema
        elsif superclass != ActiveResource::Base && (superclass.load_remote_schema != nil)
          superclass.load_remote_schema
        else
          @load_remote_schema = false
        end
      end

      def load_remote_schema=(value)
        @load_remote_schema = value
      end

      def ensure_remote_schema
        fetch_remote_schema if load_remote_schema
      end

      def map_attribute_type(association, obj)
        type = obj['type']

        case type
        when 'uniform_models_collection'
          if obj['schema'] && obj['schema']['type']
            k = find_or_create_resource_by_name(obj['schema']['type'])
            k.remote_schema = obj['schema']
            k.process_remote_schema

            has_many association, :class_name => k.name


            :embedded_collection
          end

        when 'uniform_references_collection'
          if obj['referenced_class']
            k = find_or_create_resource_by_name(obj['referenced_class'])
            k.load_remote_schema = true
          end
            has_many association, :class_name => k.name
            :collection
        else

          ActiveResource::Schema::KNOWN_ATTRIBUTE_TYPES.include?(type) ? type : :string
        end

        # TODO take care of single object associations

      end


      def process_remote_schema
        klass = self

        schema do
          klass.remote_schema['attrs'].each do |attr_key, attr_value|
            attribute attr_key.to_sym, klass.map_attribute_type(attr_key, attr_value)
          end
        end
      end

      private

      def fetch_remote_schema
        return remote_schema if remote_schema

        self.remote_schema = get(:schema).freeze
        process_remote_schema
      rescue ActiveResource::ResourceNotFound
        self.remote_schema = {}
      end

    end

  end
end