
module Ygg
  module Core
    class Person < ActiveResource::Hel::Resource
      attr_accessor :active_identity

      class << self
        def find_by_fqda(qualified)
          identity = Identity.find_by_qualified(qualified)
          person = find_by_identity(identity)
        end
        alias_method :find_by_qualified, :find_by_fqda

        def find_by_identity(identity)
          return nil unless (identity && identity.person_id)
          person = find(identity.person_id)
          person.active_identity = identity
          person
        end

        def find_by_identity_id(id)
          identity = Identity.find(id)
          find_by_identity(identity)
        end
      end

      def qualified
        active_identity.qualified if active_identity
      end

      def full_name
        return nil unless name
        [name.first, name.middle, name.last].reject{ |a| a.nil? }.join(' ')
      end

      def authenticated?(secret)
        return nil unless active_identity
        active_identity.authenticated?(secret)
      end

      def capabilities
        active_identity.caps
      end

      def has_capability?(cap)
        active_identity.has_capability?(cap)
      end

      def is_admin?
        has_capability?(:admin)
      end

      def emails
        emails = channels.find_all { |c| c.channel_type == 'email' }.sort_by { |c| c.preference }.
            map {|c| c.value }
      end

      def email
        emails.first
      end

      def attrs_for_oauth
        data = filtered_attributes(
            :uuid,
            :title,
            :first_name, :middle_name, :last_name,
            :handle,
            :birth_date,
        )
        data.merge!(:emails => emails, :email => email,
                    :identity => active_identity.attrs_for_oauth)
      end

    end
  end
end
